import { CompareFileHandler } from '../../types';
/**
 * Compare files line by line with options to ignore
 * line endings and white space differences.
 */
export declare const lineBasedFileCompare: CompareFileHandler;
//# sourceMappingURL=lineBasedFileCompare.d.ts.map