/**
 * One of 'missing','file','directory','broken-link'
 */
export function getType(entry: any): "file" | "broken-link" | "missing" | "directory";
/**
 * One of 'missing','file','directory','broken-link'
 */
export function getType(entry: any): "file" | "broken-link" | "missing" | "directory";
//# sourceMappingURL=entryType.d.ts.map