.mpv-videoPlayerContainer {
    position: fixed !important;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    display: flex;
    align-items: center;
}

.mpv-videoPlayer {
    width: 100%;
    height: 100%;
    margin: 0 !important;
    padding: 0 !important;
}

html:not(.transparentDocument) .mpv-videoPlayer {
    opacity: 0 !important;
}
