export = compare;
/**
 * Compares two directories synchronously.
 */
declare function compare(rootEntry1: any, rootEntry2: any, level: any, relativePath: any, options: any, statistics: any, diffSet: any, symlinkCache: any): void;
//# sourceMappingURL=compareSync.d.ts.map