import { Entry, PermissionDeniedState } from "../types";
export declare function getPermissionDeniedState(entry1: Entry, entry2: Entry): PermissionDeniedState;
export declare function getPrmissionDenieStateWhenLeftMissing(entry2: Entry): PermissionDeniedState;
export declare function getPrmissionDenieStateWhenRightMissing(entry1: Entry): PermissionDeniedState;
//# sourceMappingURL=permissionDeniedState.d.ts.map