export declare class Identity {
    name: string;
    hash?: string | undefined;
    constructor(name: string, hash?: string | undefined);
}
export declare function findIdentities(keychain: string | null, identity: string): Promise<Identity[]>;
