declare function _exports(entry1: any, entry2: any, state: any, level: any, relativePath: any, options: any, statistics: any, diffSet: any, reason: any, permissionDeniedState: any): void;
export = _exports;
//# sourceMappingURL=defaultResultBuilderCallback.d.ts.map