export function readdir(path: any): Promise<any>;
export function readdir(path: any): Promise<any>;
export function read(fd: any, buffer: any, offset: any, length: any, position: any): Promise<any>;
export function read(fd: any, buffer: any, offset: any, length: any, position: any): Promise<any>;
//# sourceMappingURL=fsPromise.d.ts.map