"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.d = void 0;
const debug_1 = __importDefault(require("debug"));
exports.d = debug_1.default('electron-universal');
//# sourceMappingURL=debug.js.map